@@ -32,10 +32,6 @@
 #include <utility>
 #include <variant>
 
-#include "absl/base/thread_annotations.h"
-#include "absl/container/flat_hash_map.h"
-#include "absl/random/random.h"
-#include "absl/status/status.h"
 #include "src/core/call/metadata_batch.h"  // IWYU pragma: keep
 #include "src/core/channelz/property_list.h"
 #include "src/core/ext/transport/chaotic_good_legacy/chaotic_good_transport.h"
@@ -61,6 +57,10 @@
 #include "src/core/lib/transport/promise_endpoint.h"
 #include "src/core/lib/transport/transport.h"
 #include "src/core/util/sync.h"
+#include "absl/base/thread_annotations.h"
+#include "absl/container/flat_hash_map.h"
+#include "absl/random/random.h"
+#include "absl/status/status.h"
 
 namespace grpc_core {
 namespace chaotic_good_legacy {
@@ -83,6 +83,8 @@ class ChaoticGoodClientTransport final : public ClientTransport {
   RefCountedPtr<channelz::SocketNode> GetSocketNode() const override {
     return socket_node_;
   }
+  void StartWatch(RefCountedPtr<StateWatcher> watcher) override;
+  void StopWatch(RefCountedPtr<StateWatcher> watcher) override;
 
   void StartCall(CallHandler call_handler) override;
   void AbortWithError();
@@ -134,6 +136,7 @@ class ChaoticGoodClientTransport final : public ClientTransport {
   RefCountedPtr<Party> party_;
   ConnectivityStateTracker state_tracker_ ABSL_GUARDED_BY(mu_){
       "chaotic_good_client", GRPC_CHANNEL_READY};
+  RefCountedPtr<StateWatcher> watcher_ ABSL_GUARDED_BY(mu_);
   MessageChunker message_chunker_;
   RefCountedPtr<channelz::SocketNode> socket_node_;
   std::unique_ptr<ChannelzDataSource> channelz_data_source_;
