@@ -14,19 +14,17 @@
 // limitations under the License.
 //
 
-#include <grpc/support/port_platform.h>
-
 #include "src/core/load_balancing/address_filtering.h"
 
+#include <grpc/support/port_platform.h>
 #include <stddef.h>
 
 #include <utility>
 
-#include "absl/functional/function_ref.h"
-
 #include "src/core/lib/channel/channel_args.h"
-#include "src/core/lib/gprpp/ref_counted_ptr.h"
 #include "src/core/lib/iomgr/resolved_address.h"
+#include "src/core/util/ref_counted_ptr.h"
+#include "absl/functional/function_ref.h"
 
 namespace grpc_core {
 
